package it.unibo.stradivarius.wifip2p;

import androidx.appcompat.app.AppCompatActivity;
import androidx.core.content.ContextCompat;

import android.Manifest;
import android.content.BroadcastReceiver;
import android.content.Context;
import android.content.IntentFilter;
import android.content.pm.PackageManager;
import android.net.wifi.p2p.WifiP2pConfig;
import android.net.wifi.p2p.WifiP2pDevice;
import android.net.wifi.p2p.WifiP2pDeviceList;
import android.net.wifi.p2p.WifiP2pManager;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.Button;
import android.widget.EditText;
import android.widget.TextView;

import java.io.IOException;
import java.io.OutputStream;
import java.net.InetSocketAddress;
import java.net.Socket;
import java.util.ArrayList;

public class MainActivity extends AppCompatActivity implements WifiP2pManager.PeerListListener {

    EditText etMyName;
    TextView tvRemoteNameDynamic;
    Button btnGetRemoteName;

    WifiP2pManager mManager;
    WifiP2pManager.Channel mChannel;
    BroadcastReceiver mReceiver;
    IntentFilter mIntentFilter;

    WifiP2pDevice mDevice;

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        mManager = (WifiP2pManager) getSystemService(Context.WIFI_P2P_SERVICE);
        mChannel = mManager.initialize(this, getMainLooper(), null);
        mReceiver = new MyBroadcastReceiver(mManager, mChannel, this);

        mIntentFilter = new IntentFilter();
        mIntentFilter.addAction(WifiP2pManager.WIFI_P2P_STATE_CHANGED_ACTION);
        mIntentFilter.addAction(WifiP2pManager.WIFI_P2P_PEERS_CHANGED_ACTION);
        mIntentFilter.addAction(WifiP2pManager.WIFI_P2P_CONNECTION_CHANGED_ACTION);
        mIntentFilter.addAction(WifiP2pManager.WIFI_P2P_THIS_DEVICE_CHANGED_ACTION);

        etMyName = (EditText)findViewById(R.id.etMyName);
        tvRemoteNameDynamic = (TextView)findViewById(R.id.tvRemoteNameDynamic);
        btnGetRemoteName = (Button)findViewById(R.id.btnGetRemoteName);

        mManager.discoverPeers(mChannel, new WifiP2pManager.ActionListener() {
            @Override
            public void onSuccess() {
                Log.v("MY_WIFI_P2P","DISCOVERING SUCCESS");
            }

            @Override
            public void onFailure(int reasonCode) {
                Log.v("MY_WIFI_P2P","DISCOVERING FAILURE");
            }
        });

        if (ContextCompat.checkSelfPermission(this, Manifest.permission.ACCESS_WIFI_STATE)
                != PackageManager.PERMISSION_GRANTED) {
            requestPermissions(new String[]{Manifest.permission.ACCESS_WIFI_STATE},
                    0);
        }

        BackgroundListener bl = new BackgroundListener(getApplicationContext(), tvRemoteNameDynamic);


        btnGetRemoteName.setOnClickListener(new View.OnClickListener() {
            @Override
            public void onClick(View v) {
                if (mDevice == null) {
                    tvRemoteNameDynamic.setText("No peers");
                    return;
                }
                Context context = getApplicationContext();
                String host = mDevice.deviceAddress;
                int port = 8888;
                Socket socket = new Socket();
                byte buf[]  = new byte[1024];

                try {
                    socket.bind(null);
                    socket.connect((new InetSocketAddress(host, port)), 500);

                    OutputStream outputStream = socket.getOutputStream();
                    String myName = etMyName.getText().toString();
                    byte[] allBytes = myName.getBytes();
                    for (int i = 0; i < allBytes.length; i++) {
                        outputStream.write(buf, 0, allBytes[i]);
                    }
                    outputStream.close();
                } catch (IOException e) {
                    //catch logic
                } finally {
                    if (socket != null) {
                        if (socket.isConnected()) {
                            try {
                                socket.close();
                            } catch (IOException e) {
                                //catch logic
                            }
                        }
                    }
                }
            }
        });

    }

    @Override
    protected void onResume() {
        super.onResume();
        registerReceiver(mReceiver, mIntentFilter);
        checkPermissions(Manifest.permission.ACCESS_WIFI_STATE);
        checkPermissions(Manifest.permission.CHANGE_WIFI_STATE);
        checkPermissions(Manifest.permission.CHANGE_NETWORK_STATE);
        checkPermissions(Manifest.permission.INTERNET);
        checkPermissions(Manifest.permission.ACCESS_NETWORK_STATE);
    }

    @Override
    protected void onPause() {
        super.onPause();
        unregisterReceiver(mReceiver);
    }

    @Override
    public void onPeersAvailable(WifiP2pDeviceList peers) {
        Log.v("MY_WIFI_P2P","We have " + String.valueOf(peers.getDeviceList().size()) + " peers");
        WifiP2pConfig mConfig = new WifiP2pConfig();
        ArrayList<WifiP2pDevice> list = new ArrayList<WifiP2pDevice>(peers.getDeviceList());
        mDevice = list.get(0);
        mConfig.deviceAddress = mDevice.deviceAddress;

        mManager.connect(mChannel, mConfig, new WifiP2pManager.ActionListener() {
            @Override
            public void onSuccess() {
                //success logic
            }

            @Override
            public void onFailure(int reason) {
                //failure logic
            }
        });
    }

    private void checkPermissions(String myPermission) {
        if (ContextCompat.checkSelfPermission(this, myPermission)
                != PackageManager.PERMISSION_GRANTED) {
            requestPermissions(new String[]{myPermission},
                    0);
        }
    }
}
